import type { FormKitTypeDefinition } from '@formkit/core';
import type { FormKitInputs } from '@formkit/inputs';
import { createSection, label, outer } from '@formkit/inputs';
import { markRaw } from 'vue';
import FileUploadComponent from '../components/file-upload/file-upload.vue';
import type { CustomRequest, PropFilesToValue, PropValueToFiles } from '../components/file-upload/types';
import { floatLabel } from '../sections/floatLabel';
import { help } from '../sections/help';
import { messages } from '../sections/messages';

const input = createSection('input', () => ({
  $cmp: markRaw(FileUploadComponent) as never,
  bind: '$attrs',
  props: {
    context: '$node.context',
  },
}));

export const PFileUpload: FormKitTypeDefinition = {
  type: 'input',
  schema: outer(
    floatLabel(
      input(), //
      label('$label'),
    ),
    help('$help'),
    messages(),
  ),
  props: ['fileLimit', 'maxFileSize', 'customRequest', 'valueToFiles', 'filesToValue', 'autoUpload'],
  schemaMemoKey: 'ihcxd4qdgh7', // Math.random().toString(36).substring(2, 15)
};

declare module '@formkit/inputs' {
  // https://formkit.com/essentials/custom-inputs#typescript-support
  interface FormKitInputProps<Props extends FormKitInputs<Props>> {
    PFileUpload: {
      type: 'PFileUpload';
      value?: unknown;
      fileLimit?: number;
      maxFileSize?: number;
      customRequest?: CustomRequest;
      valueToFiles?: PropValueToFiles;
      filesToValue?: PropFilesToValue;
      autoUpload?: boolean;
    };
  }
}
